# -*- coding: utf-8 -*- #
# frozen_string_literal: true

# DO NOT EDIT
# This file is automatically generated by `rake builtins:gherkin`.
# See tasks/builtins/gherkin.rake for more info.

module Rouge
  module Lexers
    class Gherkin
      # [jneen] this giant regex is staying, because Gherkin step
      # keywords are so arbitrary that there is not a good simple
      # pattern to match all of them and check post-hoc.
      STEP_REGEX = %r('a\b|'ach\b|'ej\b|\*|7\b|A\b|A\ taktiež\b|A\ také\b|A\ tiež\b|A\ zároveň\b|AN\b|Aber\b|Ac\b|Ach\b|Adott\b|Agus\b|Ak\b|Akkor\b|Alavez\b|Ale\b|Aleshores\b|Ali\b|All\ git\ out\b|Allora\b|Alors\b|Als\b|Ama\b|Amennyiben\b|Amikor\b|Amma\b|Ampak\b|An\b|Ananging\b|Ancaq\b|And\b|Angenommen\b|Anrhegedig\ a\b|Ansin\b|Antonces\b|Apabila\b|Atesa\b|Atunci\b|Atès\b|Avast!|Aye\b|BUT\b|Bagi\b|Banjur\b|Belgilangan\b|Bet\b|Bila\b|Biết\b|Blimey!|Buh\b|But\b|But\ at\ the\ end\ of\ the\ day\ I\ reckon\b|Bæþsealf\b|Bæþsealfa\b|Bæþsealfe\b|Cal\b|Cand\b|Cando\b|Ce\b|Cho\b|Ciricæw\b|Ciricæwa\b|Ciricæwe\b|Come\ hell\ or\ high\ water\b|Cuan\b|Cuando\b|Cuir\ i\ gcás\ go\b|Cuir\ i\ gcás\ gur\b|Cuir\ i\ gcás\ nach\b|Cuir\ i\ gcás\ nár\b|Când\b|DEN\b|DaH\ ghu'\ bejlu'|Dada\b|Dadas\b|Dadena\b|Dadeno\b|Dado\b|Dados\b|Daes\b|Dan\b|Dann\b|Dano\b|Daos\b|Dar\b|Dat\ fiind\b|Data\b|Date\b|Date\ fiind\b|Dati\b|Dati\ fiind\b|Dato\b|Dată\ fiind\b|Dau\b|Daus\b|Daţi\ fiind\b|Dați\ fiind\b|De\b|Den\ youse\ gotta\b|Dengan\b|Diasumsikan\b|Diberi\b|Diketahui\b|Diyelim\ ki\b|Do\b|Donada\b|Donat\b|Donc\b|Donitaĵo\b|Dun\b|Duota\b|Dáu\b|E\b|Ed\b|Eeldades\b|Ef\b|En\b|Entao\b|Entonces\b|Então\b|Entón\b|Entós\b|Epi\b|Et\b|Et\ qu'|Et\ que\b|Etant\ donné\b|Etant\ donné\ qu'|Etant\ donné\ que\b|Etant\ donnée\b|Etant\ données\b|Etant\ donnés\b|Eğer\ ki\b|Fakat\b|Fixin'\ to\b|Gangway!|Gdy\b|Gegeben\ sei\b|Gegeben\ seien\b|Gegeven\b|Gegewe\b|Gitt\b|Given\b|Givet\b|Givun\b|Ha\b|Həm\b|I\b|I\ CAN\ HAZ\b|In\b|Ir\b|It's\ just\ unbelievable\b|Ja\b|Jeśli\b|Jeżeli\b|Jika\b|Kad\b|Kada\b|Kadar\b|Kai\b|Kaj\b|Když\b|Kemudian\b|Ketika\b|Keď\b|Khi\b|Kiedy\b|Ko\b|Koga\b|Komence\b|Kui\b|Kuid\b|Kun\b|Lan\b|Le\b|Le\ sa\ a\b|Let\ go\ and\ haul\b|Logo\b|Lorsqu'|Lorsque\b|Lè\b|Lè\ sa\ a\b|Ma\b|Maar\b|Mais\b|Mais\ qu'|Mais\ que\b|Majd\b|Mając\b|Maka\b|Manawa\b|Mas\b|Men\b|Menawa\b|Mutta\b|Nalika\b|Nalikaning\b|Nanging\b|Nato\b|Nhưng\b|Niin\b|Njuk\b|No\b|Nuair\ a\b|Nuair\ ba\b|Nuair\ nach\b|Nuair\ nár\b|När\b|Når\b|Nə\ vaxt\ ki\b|O\ halda\b|O\ zaman\b|Och\b|Og\b|Oletetaan\b|Ond\b|Onda\b|Oraz\b|Pak\b|Pero\b|Peru\b|Però\b|Podano\b|Pokiaľ\b|Pokud\b|Potem\b|Potom\b|Privzeto\b|Pryd\b|Quan\b|Quand\b|Quando\b|Quick\ out\ of\ the\ chute\b|Sachant\b|Sachant\ qu'|Sachant\ que\b|Se\b|Sed\b|Si\b|Siis\b|Sipoze\b|Sipoze\ Ke\b|Sipoze\ ke\b|Soit\b|Stel\b|Så\b|Tad\b|Tada\b|Tak\b|Takrat\b|Tapi\b|Ter\b|Tetapi\b|Tha\b|Tha\ the\b|Then\b|There’s\ no\ tree\ but\ bears\ some\ fruit\b|Thurh\b|Thì\b|Toda\b|Togash\b|Too\ right\b|Tutaq\ ki\b|Ukoliko\b|Un\b|Und\b|Ve\b|Vendar\b|Verilir\b|Và\b|Və\b|WEN\b|Wanneer\b|Well\ now\ hold\ on,\ I'll\ you\ what\b|Wenn\b|When\b|Wtedy\b|Wun\b|Y\b|Y'know\b|Ya\b|Yeah\ nah\b|Yna\b|Youse\ know\ like\ when\b|Youse\ know\ when\ youse\ got\b|Za\ date\b|Za\ dati\b|Za\ dato\b|Za\ predpokladu\b|Za\ předpokladu\b|Zadan\b|Zadani\b|Zadano\b|Zakładając\b|Zakładając,\ że\b|Zaradi\b|Zatim\b|a\b|an\b|awer\b|dann\b|ghu'\ noblu'|latlh\b|mä\b|qaSDI'|ugeholl\b|vaj\b|wann\b|És\b|Étant\ donné\b|Étant\ donné\ qu'|Étant\ donné\ que\b|Étant\ donnée\b|Étant\ données\b|Étant\ donnés\b|Ða\b|Ða\ ðe\b|Ðurh\b|Þa\b|Þa\ þe\b|Þegar\b|Þurh\b|Þá\b|Če\b|Şi\b|Əgər\b|Și\b|Όταν\b|Αλλά\b|Δεδομένου\b|Και\b|Τότε\b|І\b|Інакш\b|А\b|А\ також\b|Агар\b|Але\b|Али\b|Аммо\b|Анх\b|Бирок\b|Ва\b|Вә\b|Гэхдээ\b|Дадена\b|Дадено\b|Дадзена\b|Дано\b|Допустим\b|Ды\b|Если\b|За\ дате\b|За\ дати\b|За\ дато\b|Затем\b|И\b|Иначе\b|К\ тому\ же\b|Кад\b|Када\b|Калі\b|Кога\b|Когато\b|Когда\b|Коли\b|Лекин\b|Ләкин\b|Мөн\b|Нехай\b|Но\b|Няхай\b|Нәтиҗәдә\b|Онда\b|Припустимо\b|Припустимо,\ що\b|Пусть\b|Та\b|Тады\b|Также\b|Таксама\b|То\b|Тогаш\b|Тогда\b|Тоді\b|Тэгэхэд\b|Тэгээд\b|Унда\b|Харин\b|Хэрэв\b|Якщо\b|Үүний\ дараа\b|Һәм\b|Әгәр\b|Әйтик\b|Әмма\b|Өгөгдсөн\ нь\b|Ապա\b|Բայց\b|Դիցուք\b|Եթե\b|Եվ\b|Երբ\b|אבל\b|אז\b|אזי\b|בהינתן\b|וגם\b|כאשר\b|آنگاه\b|اذاً\b|اما\b|اور\b|اگر\b|با\ فرض\b|بالفرض\b|بفرض\b|تب\b|ثم\b|جب\b|عندما\b|فرض\ کیا\b|لكن\b|لیکن\b|متى\b|هنگامی\b|و\b|پھر\b|अगर\b|अनि\b|अनी\b|आणि\b|और\b|कदा\b|किन्तु\b|चूंकि\b|जब\b|जर\b|जेव्हा\b|तथा\b|तदा\b|तब\b|तर\b|तसेच\b|तेव्हा\b|त्यसपछि\b|दिइएको\b|दिएको\b|दिलेल्या\ प्रमाणे\b|पण\b|पर\b|परंतु\b|परन्तु\b|मग\b|यदि\b|र\b|ਅਤੇ\b|ਜਦੋਂ\b|ਜਿਵੇਂ\ ਕਿ\b|ਜੇਕਰ\b|ਤਦ\b|ਪਰ\b|અને\b|આપેલ\ છે\b|ક્યારે\b|પછી\b|પણ\b|அப்பொழுது\b|ஆனால்\b|எப்போது\b|கொடுக்கப்பட்ட\b|மற்றும்\b|மேலும்\b|అప్పుడు\b|ఈ\ పరిస్థితిలో\b|కాని\b|చెప్పబడినది\b|మరియు\b|ಆದರೆ\b|ನಂತರ\b|ನೀಡಿದ\b|ಮತ್ತು\b|ಸ್ಥಿತಿಯನ್ನು\b|എപ്പോൾ\b|ഒപ്പം\b|നൽകിയത്\b|പക്ഷേ\b|പിന്നെ\b|กำหนดให้\b|ดังนั้น\b|เมื่อ\b|แต่\b|และ\b|ასევე\b|და\b|ვთქვათ\b|თუ\b|თუმცა\b|მაგრამ\b|მაშინ\b|მოცემული\b|მოცემულია\b|როგორც\ კი\b|როდესაც\b|როცა\b|መቼ\b|እና\b|ከዚያ\b|የተሰጠ\b|ግን\b|かつ\b|しかし\b|ただし\b|ならば\b|もし\b|且つ\b|並且\b|但し\b|但是\b|假如\b|假定\b|假設\b|假设\b|前提\b|同时\b|同時\b|并且\b|当\b|然し\b|當\b|而且\b|那么\b|那麼\b|그러면\b|그리고\b|단\b|만약\b|만일\b|먼저\b|조건\b|하지만\b|🎬|😂|😐|😔|🙏)o

      KEYWORDS = {}.tap do |k|
        k[:element] = Set.new ["Abstract Scenario", "Abstrakt Scenario", "Achtergrond", "Aer", "Agtergrond", "All hat and no cattle", "Antecedentes", "Antecedents", "Atburðarás", "Aturan", "Awww, look mate", "B4", "Background", "Baggrund", "Bakgrund", "Bakgrunn", "Bakgrunnur", "Beispiel", "Beispill", "Busy as a hound in flea season", "Bối cảnh", "Caso", "Casu", "Cefndir", "Cenario", "Cenario de Fundo", "Cenário", "Cenário de Fundo", "Contesto", "Context", "Contexte", "Contexto", "Cás", "Cás Achomair", "Cúlra", "Cцэнар", "Dasar", "Delineacao do Cenario", "Delineação do Cenário", "Dis is what went down", "Dyagram Senaryo", "Dyagram senaryo", "Eixemplo", "Ejemplo", "Eksempel", "Ekzemplo", "Enghraifft", "Esbozo do escenario", "Esbozu del casu", "Escenari", "Escenario", "Esempio", "Esquema de l'escenari", "Esquema del caso", "Esquema del escenario", "Esquema do Cenario", "Esquema do Cenário", "Example", "Exemple", "Exemplo", "Exemplu", "First off", "Fono", "Forgatókönyv", "Forgatókönyv vázlat", "Fundo", "Garis Panduan Senario", "Garis-Besar Skenario", "Geçmiş", "Grundlage", "Hannergrond", "Heave to", "Hintergrund", "Háttér", "Istorik", "Juhtum", "Kazo", "Kazo-skizo", "Keadaan", "Kerangka Keadaan", "Kerangka Senario", "Kerangka Situasi", "Keçmiş", "Khung kịch bản", "Khung tình huống", "Koncept", "Konsep skenario", "Kontekst", "Kontekstas", "Konteksts", "Kontext", "Konturo de la scenaro", "Kontèks", "Kural", "Kịch bản", "Latar Belakang", "Lemme tell y'all a story", "Lýsing Atburðarásar", "Lýsing Dæma", "MISHUN", "MISHUN SRSLY", "Na primer", "Náčrt Scenára", "Náčrt Scenáru", "Náčrt Scénáře", "Nümunə", "Oris scenarija", "Osnova", "Osnova Scenára", "Osnova scénáře", "Osnutek", "Ozadje", "Pavyzdys", "Piemērs", "Plan Senaryo", "Plan du Scénario", "Plan du scénario", "Plan senaryo", "Plang vum Szenario", "Pozadie", "Pozadina", "Pozadí", "Pravidlo", "Pravilo", "Pregled na scenarija", "Primer", "Primjer", "Przykład", "Príklad", "Példa", "Příklad", "Quy tắc", "Raamjuhtum", "Raamstsenaarium", "Reckon it's like", "Reegel", "Reel", "Regel", "Regla", "Regla de negocio", "Regola", "Regra", "Regulo", "Reguła", "Rerefons", "Reël", "Riail", "Rule", "Règle", "Sampla", "Scenarie", "Scenarij", "Scenarijaus šablonas", "Scenariju", "Scenariju-obris", "Scenarijus", "Scenario", "Scenario Amlinellol", "Scenario Outline", "Scenario Template", "Scenario-outline", "Scenariomal", "Scenariomall", "Scenariu", "Scenariusz", "Scenaro", "Scenár", "Scenārijs", "Scenārijs pēc parauga", "Schema dello scenario", "Scénario", "Scénář", "Senario", "Senaryo", "Senaryo Deskripsyon", "Senaryo deskripsyon", "Senaryo taslağı", "Serious as a snake bite", "Shiver me timbers", "Situasi", "Situasie", "Situasie Uiteensetting", "Situācija", "Skenario", "Skenario konsep", "Skica", "Skizo", "Sodrzhina", "Ssenari", "Ssenarinin strukturu", "Structura scenariu", "Structură scenariu", "Struktura scenarija", "Stsenaarium", "Swa", "Swa hwaer swa", "Swa hwær swa", "Szablon scenariusza", "Szabály", "Szenarien", "Szenario", "Szenariogrundriss", "Tapaus", "Tapausaihio", "Taust", "Tausta", "The thing of it is", "Tình huống", "Voorbeeld", "Voraussetzungen", "Vorbedingungen", "Wharrimean is", "Yo-ho-ho", "Zasada", "Założenia", "lut", "lut chovnatlh", "mo'", "Ær", "Örnek", "Παράδειγμα", "Περίγραμμα Σεναρίου", "Περιγραφή Σεναρίου", "Σενάριο", "Υπόβαθρο", "Агуулга", "Кантэкст", "Кереш", "Контекст", "Концепт", "На пример", "Основа", "Передумова", "Позадина", "Правило", "Правілы", "Преглед на сценарија", "Предистория", "Предыстория", "Приклад", "Пример", "Рамка на сценарий", "Скица", "Содржина", "Структура сценария", "Структура сценарија", "Структура сценарію", "Сценар", "Сценарий", "Сценарий структураси", "Сценарийның төзелеше", "Сценарио", "Сценарын төлөвлөгөө", "Сценарій", "Сцэнарый", "Тарих", "Узор сцэнара", "Шаблон сценария", "Шаблон сцэнарыя", "Կոնտեքստ", "Սցենար", "Սցենարի կառուցվացքը", "Օրինակ", "דוגמא", "כלל", "רקע", "תבנית תרחיש", "תרחיש", "الخلفية", "الگوی سناریو", "زمینه", "سناریو", "سيناريو", "سيناريو مخطط", "مثال", "منظر نامے کا خاکہ", "منظرنامہ", "پس منظر", "नियम", "परिदृश्य", "परिदृश्य रूपरेखा", "पार्श्वभूमी", "पृष्ठभूमि", "पृष्ठभूमी", "ਉਦਾਹਰਨ", "ਪਟਕਥਾ", "ਪਟਕਥਾ ਢਾਂਚਾ", "ਪਟਕਥਾ ਰੂਪ ਰੇਖਾ", "ਪਿਛੋਕੜ", "ઉદાહરણ", "નિયમ", "પરિદ્દશ્ય ઢાંચો", "પરિદ્દશ્ય રૂપરેખા", "બેકગ્રાઉન્ડ", "સ્થિતિ", "உதாரணமாக", "காட்சி", "காட்சி சுருக்கம்", "காட்சி வார்ப்புரு", "பின்னணி", "ఉదాహరణ", "కథనం", "నేపథ్యం", "సన్నివేశం", "ಉದಾಹರಣೆ", "ಕಥಾಸಾರಾಂಶ", "ವಿವರಣೆ", "ಹಿನ್ನೆಲೆ", "നിയമം", "പശ്ചാത്തലം", "രംഗം", "സാഹചര്യത്തിന്റെ രൂപരേഖ", "สรุปเหตุการณ์", "เหตุการณ์", "แนวคิด", "โครงสร้างของเหตุการณ์", "კონტექსტი", "მაგ", "მაგალითად", "მაგალითი", "ნიმუში", "სცენარი", "სცენარის ნიმუში", "სცენარის შაბლონი", "შაბლონი", "წესი", "ሁናቴ", "ሁናቴ አብነት", "ሁናቴ ዝርዝር", "ህግ", "መነሻ", "መነሻ ሀሳብ", "ምሳሌ", "ቅድመ ሁኔታ", "シナリオ", "シナリオアウトライン", "シナリオテンプレ", "シナリオテンプレート", "テンプレ", "ルール", "剧本", "剧本大纲", "劇本", "劇本大綱", "场景", "场景大纲", "場景", "場景大綱", "背景", "规则", "규칙", "배경", "시나리오", "시나리오 개요", "💤", "📕", "📖", "🥒"]
        k[:examples] = Set.new ["Atburðarásir", "Beispiele", "Beispiller", "Cenarios", "Cenários", "Conto", "Contoh", "Contone", "Dead men tell no tales", "Dæmi", "Dữ liệu", "EXAMPLZ", "Egzanp", "Eixemplos", "Ejemplos", "Eksempler", "Ekzemploj", "Enghreifftiau", "Esempi", "Examples", "Exempel", "Exemple", "Exemples", "Exemplos", "Juhtumid", "Misal", "Now that's a story longer than a cattle drive in July", "Nümunələr", "Paraugs", "Pavyzdžiai", "Piemēri", "Primeri", "Primjeri", "Przykłady", "Príklady", "Példák", "Příklady", "Samplaí", "Scenaria", "Scenarijai", "Scenariji", "Scenarios", "Se the", "Se ðe", "Se þe", "Tapaukset", "Variantai", "Voorbeelde", "Voorbeelden", "You'll wanna", "ghantoH", "lutmey", "Örnekler", "Παραδείγματα", "Σενάρια", "Значения", "Мисаллар", "Мисоллар", "Приклади", "Примери", "Примеры", "Прыклады", "Сценарија", "Сценарији", "Тухайлбал", "Үрнәкләр", "Օրինակներ", "דוגמאות", "امثلة", "مثالیں", "نمونه ها", "उदाहरण", "उदाहरणहरु", "ਉਦਾਹਰਨਾਂ", "ઉદાહરણો", "எடுத்துக்காட்டுகள்", "காட்சிகள்", "நிலைமைகளில்", "ఉదాహరణలు", "ಉದಾಹರಣೆಗಳು", "ഉദാഹരണങ്ങൾ", "ชุดของตัวอย่าง", "ชุดของเหตุการณ์", "მაგალითები", "ሁናቴዎች", "ምሳሌዎች", "サンプル", "例", "例子", "예", "📓"]
        k[:feature] = Set.new ["Ability", "Abilità", "Ahoy matey!", "All gussied up", "Arwedd", "Aspekt", "Besigheid Behoefte", "Biznis potreba", "Business Need", "Caracteristica", "Característica", "Carauterística", "Egenskab", "Egenskap", "Eiginleiki", "Esigenza di Business", "Feature", "Fitur", "Fonctionnalité", "Fonksyonalite", "Funcionalidade", "Funcionalitat", "Functionalitate", "Functionaliteit", "Funcţionalitate", "Funcționalitate", "Fungsi", "Funkcia", "Funkcija", "Funkcionalitāte", "Funkcionalnost", "Funkcja", "Funksie", "Funktion", "Funktionalität", "Funktionalitéit", "Funzionalità", "Fīča", "Gné", "Hwaet", "Hwæt", "Jellemző", "Karakteristik", "Karakteristika", "Lastnost", "Mak", "Mogucnost", "Mogućnost", "Mozhnost", "Moznosti", "Možnosti", "Necesidad del negocio", "OH HAI", "Omadus", "Ominaisuus", "Osobina", "Potrzeba biznesowa", "Požadavek", "Požiadavka", "Pretty much", "Qap", "Qu'meH 'ut", "Requisito", "Savybė", "This ain’t my first rodeo", "Trajto", "Tính năng", "Vermoë", "Vlastnosť", "Właściwość", "Značilnost", "laH", "perbogh", "poQbogh malja'", "Özellik", "Özəllik", "Δυνατότητα", "Λειτουργία", "Бизнис потреба", "Могућност", "Можност", "Мөмкинлек", "Особина", "Свойство", "Фича", "Функц", "Функционал", "Функционалност", "Функциональность", "Функция", "Функцыянальнасць", "Функціонал", "Фіча", "Үзенчәлеклелек", "Հատկություն", "Ֆունկցիոնալություն", "תכונה", "خاصية", "خصوصیت", "صلاحیت", "وِیژگی", "کاروبار کی ضرورت", "रूप लेख", "विशेषता", "वैशिष्ट्य", "सुविधा", "ਖਾਸੀਅਤ", "ਨਕਸ਼ ਨੁਹਾਰ", "ਮੁਹਾਂਦਰਾ", "ક્ષમતા", "લક્ષણ", "વ્યાપાર જરૂર", "அம்சம்", "திறன்", "வணிக தேவை", "గుణము", "ಹೆಚ್ಚಳ", "സവിശേഷത", "ความต้องการทางธุรกิจ", "ความสามารถ", "โครงหลัก", "თვისება", "მოთხოვნა", "ስራ", "የሚፈለገው ድርጊት", "የተፈለገው ስራ", "フィーチャ", "功能", "機能", "기능", "📚"]
      end
    end
  end
end
